import React, { useState, useRef, useEffect, useContext, useMemo, useLayoutEffect } from 'react'
import { MenuOutlined } from '@ant-design/icons'
import { Table, Input, Form, Popover, Popconfirm } from 'antd'
import { arrayMoveImmutable } from 'array-move'
import { SortableContainer, SortableElement, SortableHandle } from 'react-sortable-hoc'
import { CheckOutlined, DeleteOutlined } from '@ant-design/icons'
import styles from './index.less'

const colors = [
  // '#ffccc7', '#ffe7ba', '#fffb8f', '#d9f7be', '#b5f5ec',
  // '#bae7ff', '#d6e4ff', '#efdbff', '#ffd6e7', '#e8e8e8',
  // '#ff4d4f', '#ffa940', '#fbce00', '#73d13d', '#36cfc9',
  // '#40a9ff', '#597ef7', '#9254de', '#f759ab', '#8c8c8c',
  'rgb(255, 77, 79)',
  'rgb(255, 169, 64)',
  'rgb(251, 206, 0)',
  'rgba(115, 209, 61)',
  'rgba(54, 207, 201)',
  'rgba(64, 169, 255)',
  'rgba(89, 126, 247)',
  'rgba(146, 84, 222)',
  'rgba(247, 89, 171)',
  'rgba(140, 140, 140)',
  // 'rgba(255, 204, 199)',
  // 'rgba(255, 231, 186)',
  // 'rgba(255, 251, 143)',
  // 'rgba(217, 247, 190)',
  // 'rgba(181, 245, 236)',
  // 'rgba(186, 231, 255)',
  // 'rgba(214, 228, 255)',
  // 'rgba(239, 219, 255)',
  // 'rgba(255, 214, 231)',
  // 'rgba(232, 232, 232)',
]

let newData = []

const EditableContext = React.createContext(null)

const DragHandle = SortableHandle(() => (
  <MenuOutlined
    style={{
      cursor: 'grab',
      color: '#999',
    }}
  />
))

const SortableItem = SortableElement((props) => {
  const [form] = Form.useForm()
  return (
    <Form form={form} component={false}>
      <EditableContext.Provider value={form}>
        <tr {...props} />
      </EditableContext.Provider>
    </Form>
  )
})

const SortableBody = SortableContainer((props) => <tbody {...props} />)

// const InputItem = (props) => {

//   const inputRef = useRef(null)

//   useLayoutEffect(() => {
//     inputRef.current.focus()
//   }, [])

//   return (
//     <Input ref={inputRef} {...props} />
//   )
// }

const DragTable = (props) => {

  const { dataSource, setDataSource, color } = props

  const onSortEnd = ({ oldIndex, newIndex }) => {
    if (oldIndex !== newIndex) {
      const data = arrayMoveImmutable(dataSource.slice(), oldIndex, newIndex).filter(
        (el) => !!el,
      )
      setDataSource(data)
    }
  }

  const DraggableContainer = (props) => (
    <SortableBody
      useDragHandle
      disableAutoscroll
      helperClass="row-dragging"
      onSortEnd={onSortEnd}
      {...props}
    />
  )

  const DraggableBodyRow = ({ className, style, ...restProps }) => {
    // const index = dataSource.findIndex((x) => x.index === restProps['data-row-key'])
    // return <SortableItem key={index} index={index} {...restProps} />
    let key = Number(restProps['data-row-key'])
    return <SortableItem key={key} index={key} {...restProps} />
  }

  const onPressEnter = (i) => {
    let length = [...dataSource].length
    let array = [...dataSource]
    array.splice(i + 1, 0, { key: `${length + 1}`, index: length + 1, value: '' })
    setDataSource(array)
  }

  const inputChange = (value, i) => {
    let array = []
    dataSource.forEach((v, index) => {
      if (index === i) {
        v.value = value
      }
      array.push({ ...v })
    })
    newData = array
  }

  const colorClick = (value, i) => {
    let array = []
    dataSource.forEach((v, index) => {
      if (index === i) {
        v.color = value
      }
      array.push({ ...v })
    })
    setDataSource(array)
  }

  const onBlur = () => {
    if (Array.isArray(newData) && newData.length) {
      setDataSource(newData)
    }
  }

  const confirm = (index) => {
    let array = []
    dataSource.forEach((v, i) => {
      if (i !== index) {
        array.push({ ...v })
      }
    })
    newData = array
    setDataSource(array)
  }

  const rest = (index) => {
    let array = [] 
    dataSource.forEach((v, i) => {
      if (index === i) {
        v.color = ''
      }
      array.push({ ...v })
    })
    setDataSource(array)
  }

  const columns = useMemo(() => {
    let columns = [
      {
        title: 'value',
        dataIndex: 'value',
        render: (r, _, i) => {
          return (
            <Input
              onPressEnter={() => onPressEnter(i)}
              onChange={(e) => inputChange(e.target.value, i)}
              defaultValue={r}
              onBlur={() => onBlur()}
            />
          )
        }
      },
      {
        title: 'color',
        dataIndex: 'color',
        width: 30,
        render: (r, _, index) => {
          const content = (
            <div>
              <div className={styles.colorTop}>
                <div className={styles.topLeft}>请选择颜色</div>
                <div className={styles.topRight} onClick={() => rest(index)}>重置</div>
              </div>
              <div className={styles.colorBox}>
                {
                  colors.map((v, i) => {
                    return (
                      <div className={styles.colors} style={{ background: v }} onClick={() => colorClick(v, index)}>
                        <CheckOutlined style={{ fontSize: '11px', marginLeft: '5px', display: r === v ? 'inline-block' : 'none' }} />
                      </div>
                    )
                  })
                }
              </div>
            </div>
          )
          return (
            <Popover
              content={content}
              trigger='click'
            >
              <div className={styles.colors} style={{ background: r || '#f2f4f5' }}></div>
            </Popover>
          )
        }
      },
      {
        width: 30,
        render: (r, _, i) => {
          return (
            <Popconfirm
              title='确定删除?'
              onConfirm={() => confirm(i)}
              onCancel={() => { }}
            >
              <DeleteOutlined style={{ color: 'rgb(153, 153, 153)' }} />
            </Popconfirm>
          )
        },
      },
      {
        title: 'Sort',
        dataIndex: 'sort',
        width: 30,
        className: 'drag-visible',
        render: () => <DragHandle />,
      },
    ]
    if (!color) {
      return columns.filter(v => v.dataIndex !== 'color')
    }
    return columns
  }, [dataSource, color])

  return (
    <Table
      showHeader={false}
      pagination={false}
      dataSource={dataSource}
      columns={columns}
      rowKey="key"
      size='small'
      components={{
        body: {
          wrapper: DraggableContainer,
          row: DraggableBodyRow,
        },
      }}
    />
  )

}

export default DragTable