import React, { useState, useRef, useContext, useEffect } from 'react'
import styles from './index.less'
import { Switch, Input, Table, Form } from 'antd'
import Drag from './../../../../components/Drag'
import DragTable from '../../../../components/DragTable'

const SimpleList = (props) => {

  const { value, onChange, addons } = props
  const { color } = addons.formData
  const [visible, setVisible] = useState(false)
  const [text, setText] = useState('')

  const switchChange = (checked) => {
    addons.setValue('color', checked)
  }

  const add = () => {
    let length = [...value].length
    let array = [...value]
    array.push({ key: `${length + 1}`, index: length + 1, value: '' })
    onChange(array)
  }

  const edit = () => {
    setText(value.map(v => v.value).join('\n'))
    setVisible(true)
  }

  const onOk = () => {
    let array = text.split('\n').map((v, i) => { return { key: `${i}`, index: i, value: v } })
    onChange(array)
    setVisible(false)
  }

  return (
    <div className={styles.simpleList}>
      <div className={styles.title}>
        <div style={{ fontSize: '14px' }}>选项</div>
        <div style={{ fontSize: '12px' }}>
          <span style={{ paddingRight: '5px' }}>彩色</span>
          <Switch size='small' checked={color} onChange={switchChange} />
        </div>
      </div>
      <div className={styles.box}>
        <DragTable
          dataSource={Array.isArray(value) ? value.map((v, i) => { return { key: `${i}`, index: i, ...v } }) : []}
          setDataSource={onChange}
          color={color}
        />
      </div>
      <div className={styles.footer}>
        <span onClick={add}>添加选项</span>
        <span></span>
        <span onClick={edit}>批量编辑</span>
      </div>
      <Drag
        title='批量编辑'
        onOk={onOk}
        onCancel={() => setVisible(false)}
        visible={visible}
      >
        <Input.TextArea
          rows={5}
          value={text}
          onChange={(e) => setText(e.target.value)}
        />
      </Drag>
    </div>
  )
}

export default SimpleList